@@ -7,7 +7,8 @@
 # For further information on the license, see the LICENSE.txt file        #
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
-
+# pylint: disable=invalid-name
+"""Database migration."""
 from django.db import migrations
 from aiida.backends.djsite.db.migrations import upgrade_schema_version
 
@@ -16,6 +17,7 @@ DOWN_REVISION = '1.0.15'
 
 
 class Migration(migrations.Migration):
+    """Database migration."""
 
     dependencies = [
         ('db', '0015_invalidating_node_hash'),
@@ -26,6 +28,7 @@ class Migration(migrations.Migration):
         # To make everything fully consistent, its type string should therefore also start with `data.`
         migrations.RunSQL(
             sql="""UPDATE db_dbnode SET type = 'data.code.Code.' WHERE type = 'code.Code.';""",
-            reverse_sql="""UPDATE db_dbnode SET type = 'code.Code.' WHERE type = 'data.code.Code.';"""),
+            reverse_sql="""UPDATE db_dbnode SET type = 'code.Code.' WHERE type = 'data.code.Code.';"""
+        ),
         upgrade_schema_version(REVISION, DOWN_REVISION)
     ]
