@@ -159,32 +159,32 @@ SETUP_PROFILE = options.OverridableOption(
 
 SETUP_USER_EMAIL = options.USER_EMAIL.clone(
     prompt='Email Address (for sharing data)',
-    default=get_config_option('user.email'),
-    required_fn=lambda x: get_config_option('user.email') is None,
+    default=get_config_option('autofill.user.email'),
+    required_fn=lambda x: get_config_option('autofill.user.email') is None,
     required=True,
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_USER_FIRST_NAME = options.USER_FIRST_NAME.clone(
     prompt='First name',
-    default=get_config_option('user.first_name'),
-    required_fn=lambda x: get_config_option('user.first_name') is None,
+    default=get_config_option('autofill.user.first_name'),
+    required_fn=lambda x: get_config_option('autofill.user.first_name') is None,
     required=True,
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_USER_LAST_NAME = options.USER_LAST_NAME.clone(
     prompt='Last name',
-    default=get_config_option('user.last_name'),
-    required_fn=lambda x: get_config_option('user.last_name') is None,
+    default=get_config_option('autofill.user.last_name'),
+    required_fn=lambda x: get_config_option('autofill.user.last_name') is None,
     required=True,
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_USER_INSTITUTION = options.USER_INSTITUTION.clone(
     prompt='Institution',
-    default=get_config_option('user.institution'),
-    required_fn=lambda x: get_config_option('user.institution') is None,
+    default=get_config_option('autofill.user.institution'),
+    required_fn=lambda x: get_config_option('autofill.user.institution') is None,
     required=True,
     cls=options.interactive.InteractiveOption
 )
