@@ -13,18 +13,24 @@ import functools
 import inspect
 import logging
 import signal
+from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Type, TYPE_CHECKING
 
 from aiida.common.lang import override
 from aiida.manage.manager import get_manager
+from aiida.orm import CalcFunctionNode, Data, ProcessNode, WorkFunctionNode
+from aiida.orm.utils.mixins import FunctionCalculationMixin
 
 from .process import Process
 
+if TYPE_CHECKING:
+    from .exit_code import ExitCode
+
 __all__ = ('calcfunction', 'workfunction', 'FunctionProcess')
 
 LOGGER = logging.getLogger(__name__)
 
 
-def calcfunction(function):
+def calcfunction(function: Callable[..., Any]) -> Callable[..., Any]:
     """
     A decorator to turn a standard python function into a calcfunction.
     Example usage:
@@ -51,11 +57,10 @@ def calcfunction(function):
     :return: The decorated function.
     :rtype: callable
     """
-    from aiida.orm import CalcFunctionNode
     return process_function(node_class=CalcFunctionNode)(function)
 
 
-def workfunction(function):
+def workfunction(function: Callable[..., Any]) -> Callable[..., Any]:
     """
     A decorator to turn a standard python function into a workfunction.
     Example usage:
@@ -80,13 +85,12 @@ def workfunction(function):
     :type function: callable
 
     :return: The decorated function.
-    :rtype: callable
-        """
-    from aiida.orm import WorkFunctionNode
+
+    """
     return process_function(node_class=WorkFunctionNode)(function)
 
 
-def process_function(node_class):
+def process_function(node_class: Type['ProcessNode']) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
     """
     The base function decorator to create a FunctionProcess out of a normal python function.
 
@@ -94,7 +98,7 @@ def process_function(node_class):
     :type node_class: :class:`aiida.orm.ProcessNode`
     """
 
-    def decorator(function):
+    def decorator(function: Callable[..., Any]) -> Callable[..., Any]:
         """
         Turn the decorated function into a FunctionProcess.
 
@@ -103,21 +107,17 @@ def process_function(node_class):
         """
         process_class = FunctionProcess.build(function, node_class=node_class)
 
-        def run_get_node(*args, **kwargs):
+        def run_get_node(*args, **kwargs) -> Tuple[Optional[Dict[str, Any]], 'ProcessNode']:
             """
             Run the FunctionProcess with the supplied inputs in a local runner.
 
-            The function will have to create a new runner for the FunctionProcess instead of using the global runner,
-            because otherwise if this process function were to call another one from within its scope, that would use
-            the same runner and it would be blocking the event loop from continuing.
-
             :param args: input arguments to construct the FunctionProcess
             :param kwargs: input keyword arguments to construct the FunctionProcess
-            :return: tuple of the outputs of the process and the process node pk
-            :rtype: (dict, int)
+            :return: tuple of the outputs of the process and the process node
+
             """
             manager = get_manager()
-            runner = manager.create_runner(with_persistence=False)
+            runner = manager.get_runner()
             inputs = process_class.create_inputs(*args, **kwargs)
 
             # Remove all the known inputs from the kwargs
@@ -140,10 +140,9 @@ def process_function(node_class):
 
                 def kill_process(_num, _frame):
                     """Send the kill signal to the process in the current scope."""
-                    from tornado import gen
                     LOGGER.critical('runner received interrupt, killing process %s', process.pid)
                     result = process.kill(msg='Process was killed because the runner received an interrupt')
-                    raise gen.Return(result)
+                    return result
 
                 # Store the current handler on the signal such that it can be restored after process has terminated
                 original_handler = signal.getsignal(kill_signal)
@@ -155,7 +154,6 @@ def process_function(node_class):
                 # If the `original_handler` is set, that means the `kill_process` was bound, which needs to be reset
                 if original_handler:
                     signal.signal(signal.SIGINT, original_handler)
-                runner.close()
 
             store_provenance = inputs.get('metadata', {}).get('store_provenance', True)
             if not store_provenance:
@@ -164,13 +162,13 @@ def process_function(node_class):
 
             return result, process.node
 
-        def run_get_pk(*args, **kwargs):
+        def run_get_pk(*args, **kwargs) -> Tuple[Optional[Dict[str, Any]], int]:
             """Recreate the `run_get_pk` utility launcher.
 
             :param args: input arguments to construct the FunctionProcess
             :param kwargs: input keyword arguments to construct the FunctionProcess
             :return: tuple of the outputs of the process and the process node pk
-            :rtype: (dict, int)
+
             """
             result, node = run_get_node(*args, **kwargs)
             return result, node.pk
@@ -181,14 +179,14 @@ def process_function(node_class):
             result, _ = run_get_node(*args, **kwargs)
             return result
 
-        decorated_function.run = decorated_function
-        decorated_function.run_get_pk = run_get_pk
-        decorated_function.run_get_node = run_get_node
-        decorated_function.is_process_function = True
-        decorated_function.node_class = node_class
-        decorated_function.process_class = process_class
-        decorated_function.recreate_from = process_class.recreate_from
-        decorated_function.spec = process_class.spec
+        decorated_function.run = decorated_function  # type: ignore[attr-defined]
+        decorated_function.run_get_pk = run_get_pk  # type: ignore[attr-defined]
+        decorated_function.run_get_node = run_get_node  # type: ignore[attr-defined]
+        decorated_function.is_process_function = True  # type: ignore[attr-defined]
+        decorated_function.node_class = node_class  # type: ignore[attr-defined]
+        decorated_function.process_class = process_class  # type: ignore[attr-defined]
+        decorated_function.recreate_from = process_class.recreate_from  # type: ignore[attr-defined]
+        decorated_function.spec = process_class.spec  # type: ignore[attr-defined]
 
         return decorated_function
 
@@ -198,10 +196,10 @@ def process_function(node_class):
 class FunctionProcess(Process):
     """Function process class used for turning functions into a Process"""
 
-    _func_args = None
+    _func_args: Sequence[str] = ()
 
     @staticmethod
-    def _func(*_args, **_kwargs):
+    def _func(*_args, **_kwargs) -> dict:
         """
         This is used internally to store the actual function that is being
         wrapped and will be replaced by the build method.
@@ -209,7 +207,7 @@ class FunctionProcess(Process):
         return {}
 
     @staticmethod
-    def build(func, node_class):
+    def build(func: Callable[..., Any], node_class: Type['ProcessNode']) -> Type['FunctionProcess']:
         """
         Build a Process from the given function.
 
@@ -217,19 +215,13 @@ class FunctionProcess(Process):
         these will also become inputs.
 
         :param func: The function to build a process from
-        :type func: callable
-
         :param node_class: Provide a custom node class to be used, has to be constructable with no arguments. It has to
             be a sub class of `ProcessNode` and the mixin :class:`~aiida.orm.utils.mixins.FunctionCalculationMixin`.
-        :type node_class: :class:`aiida.orm.nodes.process.process.ProcessNode`
 
         :return: A Process class that represents the function
-        :rtype: :class:`FunctionProcess`
-        """
-        from aiida import orm
-        from aiida.orm.utils.mixins import FunctionCalculationMixin
 
-        if not issubclass(node_class, orm.ProcessNode) or not issubclass(node_class, FunctionCalculationMixin):
+        """
+        if not issubclass(node_class, ProcessNode) or not issubclass(node_class, FunctionCalculationMixin):
             raise TypeError('the node_class should be a sub class of `ProcessNode` and `FunctionCalculationMixin`')
 
         args, varargs, keywords, defaults, _, _, _ = inspect.getfullargspec(func)
@@ -246,7 +238,7 @@ class FunctionProcess(Process):
 
             for i, arg in enumerate(args):
                 default = ()
-                if i >= first_default_pos:
+                if defaults and i >= first_default_pos:
                     default = defaults[i - first_default_pos]
 
                 # If the keyword was already specified, simply override the default
@@ -257,9 +249,9 @@ class FunctionProcess(Process):
                     # Note that we cannot use `None` because the validation will call `isinstance` which does not work
                     # when passing `None`, but it does work with `NoneType` which is returned by calling `type(None)`
                     if default is None:
-                        valid_type = (orm.Data, type(None))
+                        valid_type = (Data, type(None))
                     else:
-                        valid_type = (orm.Data,)
+                        valid_type = (Data,)
 
                     spec.input(arg, valid_type=valid_type, default=default)
 
@@ -275,7 +267,7 @@ class FunctionProcess(Process):
             # Function processes must have a dynamic output namespace since we do not know beforehand what outputs
             # will be returned and the valid types for the value should be `Data` nodes as well as a dictionary because
             # the output namespace can be nested.
-            spec.outputs.valid_type = (orm.Data, dict)
+            spec.outputs.valid_type = (Data, dict)
 
         return type(
             func.__name__, (FunctionProcess,), {
@@ -289,7 +281,7 @@ class FunctionProcess(Process):
         )
 
     @classmethod
-    def validate_inputs(cls, *args, **kwargs):  # pylint: disable=unused-argument
+    def validate_inputs(cls, *args: Any, **kwargs: Any) -> None:  # pylint: disable=unused-argument
         """
         Validate the positional and keyword arguments passed in the function call.
 
@@ -308,11 +300,8 @@ class FunctionProcess(Process):
             raise TypeError(f'{name}() takes {nparameters} positional arguments but {nargs} were given')
 
     @classmethod
-    def create_inputs(cls, *args, **kwargs):
-        """Create the input args for the FunctionProcess.
-
-        :rtype: dict
-        """
+    def create_inputs(cls, *args: Any, **kwargs: Any) -> Dict[str, Any]:
+        """Create the input args for the FunctionProcess."""
         cls.validate_inputs(*args, **kwargs)
 
         ins = {}
@@ -323,29 +312,28 @@ class FunctionProcess(Process):
         return ins
 
     @classmethod
-    def args_to_dict(cls, *args):
+    def args_to_dict(cls, *args: Any) -> Dict[str, Any]:
         """
         Create an input dictionary (of form label -> value) from supplied args.
 
         :param args: The values to use for the dictionary
-        :type args: list
 
         :return: A label -> value dictionary
-        :rtype: dict
+
         """
         return dict(list(zip(cls._func_args, args)))
 
     @classmethod
-    def get_or_create_db_record(cls):
+    def get_or_create_db_record(cls) -> 'ProcessNode':
         return cls._node_class()
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         if kwargs.get('enable_persistence', False):
             raise RuntimeError('Cannot persist a function process')
-        super().__init__(enable_persistence=False, *args, **kwargs)
+        super().__init__(enable_persistence=False, *args, **kwargs)  # type: ignore
 
     @property
-    def process_class(self):
+    def process_class(self) -> Callable[..., Any]:
         """
         Return the class that represents this Process, for the FunctionProcess this is the function itself.
 
@@ -354,33 +342,29 @@ class FunctionProcess(Process):
         class that really represents what was being executed.
 
         :return: A Process class that represents the function
-        :rtype: :class:`FunctionProcess`
+
         """
         return self._func
 
-    def execute(self):
+    def execute(self) -> Optional[Dict[str, Any]]:
         """Execute the process."""
         result = super().execute()
 
         # FunctionProcesses can return a single value as output, and not a dictionary, so we should also return that
-        if len(result) == 1 and self.SINGLE_OUTPUT_LINKNAME in result:
+        if result and len(result) == 1 and self.SINGLE_OUTPUT_LINKNAME in result:
             return result[self.SINGLE_OUTPUT_LINKNAME]
 
         return result
 
     @override
-    def _setup_db_record(self):
+    def _setup_db_record(self) -> None:
         """Set up the database record for the process."""
         super()._setup_db_record()
         self.node.store_source_info(self._func)
 
     @override
-    def run(self):
-        """Run the process.
-
-        :rtype: :class:`aiida.engine.ExitCode`
-        """
-        from aiida.orm import Data
+    def run(self) -> Optional['ExitCode']:
+        """Run the process."""
         from .exit_code import ExitCode
 
         # The following conditional is required for the caching to properly work. Even if the source node has a process
@@ -394,9 +378,9 @@ class FunctionProcess(Process):
         args = [None] * len(self._func_args)
         kwargs = {}
 
-        for name, value in self.inputs.items():
+        for name, value in (self.inputs or {}).items():
             try:
-                if self.spec().inputs[name].non_db:
+                if self.spec().inputs[name].non_db:  # type: ignore[union-attr]
                     # Don't consider non-database inputs
                     continue
             except KeyError:
