@@ -3,7 +3,7 @@
 # Copyright (c), The AiiDA team. All rights reserved.                     #
 # This file is part of the AiiDA code.                                    #
 #                                                                         #
-# The code is hosted on GitHub at https://github.com/aiidateam/aiida_core #
+# The code is hosted on GitHub at https://github.com/aiidateam/aiida-core #
 # For further information on the license, see the LICENSE.txt file        #
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
@@ -24,17 +24,21 @@ or internal errors, are not embedded into the HTTP response.
 from aiida.common.exceptions import ValidationError, InputValidationError, \
     FeatureNotAvailable
 
+
 class RestValidationError(ValidationError):
     """
-    document with an example
+    If validation error in code
+    E.g. more than one node available for given uuid
     """
-    pass
+
 
 class RestInputValidationError(InputValidationError):
     """
-    document with an example
+    If inputs passed in query strings are wrong
     """
-    pass
+
 
 class RestFeatureNotAvailable(FeatureNotAvailable):
-    pass
+    """
+    If endpoint is not emplemented for given node type
+    """
